import argparse
import json
import sys
import time
import ssl
import base64

from task_common import read_task_file
from task_common import post_task
from task_common import await_task_completion
from task_common import await_load_completion
from task_common import raise_friendly_error
from urllib.request import urlopen
from urllib.request import Request
from urllib.error import URLError
from urllib.error import HTTPError
from datetime import datetime, timedelta


def get_time_interval(number_of_day_for_replay=30):
    # rollback to 30 days
    interval_start_timestamp = datetime.today() - timedelta(days=number_of_day_for_replay)
    interval_end_timestamp = datetime.today()
    time_interval = interval_start_timestamp.strftime('%Y-%m-%dT%H:00:00') + "/" + interval_end_timestamp \
        .strftime('%Y-%m-%dT%H:00:00')
    return time_interval


'''
curl  https://druid-coordinator:8281/druid/coordinator/v1/loadstatus -H 'Content-Type:application/json' -H \
'Accept:application/json'  -H 'x-nsx-roles: nsx_base64_encoded_role' \

Response \
{
    "context_user_metadata": 100.0,
    "correlated_flow_viz": 100.0,
    "correlated_flow": 100.0,
    "pace2druid_manager_realization_config": 100.0,
    "correlated_flow_rec": 100.0,
    "pace2druid_policy_intent_config": 100.0
}
'''


def add_basic_auth_header(args, req):
    if args.user is not None:
        basic_auth_encoded = base64.b64encode('%s:%s' % (args.user, args.password))
        req.add_header("Authorization", "Basic %s" % basic_auth_encoded)


def need_upgrade(args, ctx):
    response = json.loads(simple_get_client(args, ctx))
    print("Response ", response)
    # Check coordinator has any status of viz table required for upgrade else we dont need upgrade.
    if "correlated_flow_viz" in response:
        print("This deployment need Recommendation Druid table upgrade")
        return True
    print("Druid does not contains correlated_flow_viz table.")
    return False


def simple_get_client(args, ctx=None):
    start_time = time.time()
    try:
        url = args.coordinator_url + "druid/coordinator/v1/loadstatus"
        print("Url:", url)
        req = Request(url, None, {'Content-Type': 'application/json'})
        add_basic_auth_header(args, req)
        print("Req:", str(req))
        response_timeout = 10
        response = urlopen(req, timeout=response_timeout, context=ctx)
        return response.read().rstrip()
    except URLError as e:
        if isinstance(e, HTTPError) and 400 <= e.code <= 500:
            # 4xx (problem with the request) or 500 (something wrong on the server)
            raise_friendly_error(e)
        elif time.time() >= start_time + response_timeout:
            # No further retries
            raise_friendly_error(e)
        else:
            # If at first you don't succeed, try, try again!
            sleep_time = 5
            if not args.quiet:
                extra = ''
                if hasattr(e, 'read'):
                    extra = e.read().rstrip()
                print("Waiting up to {0}s for indexing service [{1}] to become available. [Got: {2} {3}]"
                      .format(max(sleep_time, int(start_time - time.time())), args.url, str(e), extra)
                      .rstrip())
            time.sleep(sleep_time)
            return simple_get_client(args)


def main():
    parser = argparse.ArgumentParser(description='Post Druid indexing tasks.')
    parser.add_argument('--src-data-source', '-sd', type=str, default='correlated_flow_viz',
                        help='The name of Druid table (dataSource) as source of indexing')
    parser.add_argument('--dst-data-source', '-dd', type=str, default='correlated_flow_rec',
                        help='The name of Druid table (dataSource) as destination of indexing')
    parser.add_argument('--coordinator-url', type=str, default='https://druid-coordinator:8281/',
                        help='Druid Coordinator url')
    parser.add_argument('--url', '-u', metavar='url', type=str, default='https://druid-overlord:8290/',
                        help='Druid Overlord url')
    parser.add_argument('--file', '-f', type=str, required=True, help='Query JSON file')
    parser.add_argument('--submit-timeout', type=int, default=120,
                        help='Timeout (in seconds) for submitting tasks')
    parser.add_argument('--complete-timeout', type=int, default=14400,
                        help='Timeout (in seconds) for completing tasks')
    parser.add_argument('--load-timeout', type=int, default=14400,
                        help='Timeout (in seconds) for waiting for tasks to load')
    parser.add_argument('--quiet', '-q', action='store_true', help='Suppress retryable errors')
    parser.add_argument('--user', type=str, default=None, help='Basic auth username')
    parser.add_argument('--password', type=str, default=None, help='Basic auth password')
    args = parser.parse_args()

    srcdatasource = args.src_data_source
    dstdatasource = args.dst_data_source

    submit_timeout_at = time.time() + args.submit_timeout
    complete_timeout_at = time.time() + args.complete_timeout
    print("Input args: ", args)
    task_contents = read_task_file(args.file)
    task_json = json.loads(task_contents)
    # {
    #     "type"    : "ingestSegment",
    #     "dataSource"   : "wikipedia",
    #     "interval" : "2013-01-01/2013-01-02"
    # }
    # interval_end_timestamp = datetime.today() - timedelta(days=30)
    # update the interval for last 30 days from today
    #

    task_json["spec"]["ioConfig"]["inputSource"]["interval"] = get_time_interval()
    task_json["spec"]["ioConfig"]["inputSource"]["dataSource"] = srcdatasource
    task_json["spec"]["dataSchema"]["dataSource"] = dstdatasource
    task_contents = json.dumps(task_json)

    print("Json task after spec updates: \n", task_json)
    # SSL context initialization
    ctx = ssl.create_default_context()
    print(ctx)
    # verify that CA cert work on NS as well as on Jetfire loaded from druid certs
    ctx.load_verify_locations("/etc/nsxi/certs/ca.crt")

    if not need_upgrade(args, ctx):
        print("Found no reason to upgrade hence exiting upgrade job. Hence Exiting job")
        return 1

    task_id = json.loads(post_task(args, task_contents, submit_timeout_at, ctx))["task"]

    print('\033[1m' + "Task started: " + '\033[0m' + "{0}".format(task_id))
    print('\033[1m' + "Task log:     " + '\033[0m' +
          "{0}/druid/indexer/v1/task/{1}/log".format(args.url.rstrip("/"), task_id))
    print('\033[1m' + "Task status:  " + '\033[0m' +
          "{0}/druid/indexer/v1/task/{1}/status".format(args.url.rstrip("/"), task_id))

    task_status = await_task_completion(args, task_id, complete_timeout_at, ctx)
    print("Task finished with status: {0}".format(task_status))
    if task_status != 'SUCCESS':
        sys.exit(1)

    print("Completed indexing data for {0}. Now loading indexed data onto the cluster...".format(dstdatasource))
    load_timeout_at = time.time() + args.load_timeout
    await_load_completion(args, dstdatasource, load_timeout_at, ctx)


try:
    main()
except KeyboardInterrupt:
    sys.exit(1)
