import base64
import json
import re
import sys
import time

from urllib.request import urlopen
from urllib.request import Request
from urllib.request import quote
from urllib.error import URLError
from urllib.error import HTTPError
from urllib.parse import urlparse

def read_task_file(file_path):
  print("Starting to read the task path %d" , file_path)
  with open(file_path, 'r') as f:
    contents = f.read()
    # We don't use the parsed data, but we want to throw early if it's invalid
    try:
      json.loads(contents)
    except Exception as e:
      print('Invalid JSON in task file "{0}": {1}'.format(file_path, repr(e)), flush=True)
      sys.exit(1)
    return contents

def add_basic_auth_header(args, req):
  if (args.user is not None):
    basic_auth_encoded = base64.b64encode('%s:%s' % (args.user, args.password))
    req.add_header("Authorization", "Basic %s" % basic_auth_encoded)

# Keep trying until timeout_at, maybe die then
def post_task(args, task_json, timeout_at, ctx=None):
  try:
    url = args.url.rstrip("/") + "/druid/indexer/v1/task"
    data = task_json.encode("utf-8")
    req = Request(url, None, {'Content-Type' : 'application/json'})
    add_basic_auth_header(args, req)
    timeleft = timeout_at - time.time()
    response_timeout = min(max(timeleft, 5), 10)
    response = urlopen(req, data, response_timeout, context=ctx)
    return response.read().rstrip()
  except URLError as e:
    if isinstance(e, HTTPError) and e.code >= 400 and e.code <= 500:
      # 4xx (problem with the request) or 500 (something wrong on the server)
      raise_friendly_error(e)
    elif time.time() >= timeout_at:
      # No futher retries
      raise_friendly_error(e)
    elif isinstance(e, HTTPError) and e.code in [301, 302, 303, 305, 307] and \
      e.getheader("Location") is not None:
      # Set the new location in args.url so it can be used by await_task_completion and re-issue the request
      location = urlparse(e.getheader("Location"))
      args.url = "{0}://{1}".format(location.scheme, location.netloc)
      print("Redirect response received, setting url to [{0}]".format(args.url), flush=True)
      return post_task(args, task_json, timeout_at, ctx)
    else:
      # If at first you don't succeed, try, try again!
      sleep_time = 5
      if not args.quiet:
        extra = ''
        if hasattr(e, 'read'):
          extra = e.read().rstrip()
        print("Waiting up to {0}s for indexing service [{1}] to become available. [Got: {2} {3}]"
              .format(max(sleep_time, int(timeout_at - time.time())), args.url, str(e), extra).rstrip(), flush=True)
      time.sleep(sleep_time)
      return post_task(args, task_json, timeout_at)

# Keep trying until timeout_at, maybe die then
def await_task_completion(args, task_id, timeout_at, ctx=None):
  while True:
    url = args.url.rstrip("/") + "/druid/indexer/v1/task/{0}/status".format(quote(task_id))
    req = Request(url)
    add_basic_auth_header(args, req)
    timeleft = timeout_at - time.time()
    response_timeout = min(max(timeleft, 5), 10)
    response = urlopen(req, None, response_timeout, context=ctx)
    response_obj = json.loads(response.read())
    response_status_code = response_obj["status"]["status"]
    if response_status_code in ['SUCCESS', 'FAILED']:
      return response_status_code
    else:
      if time.time() < timeout_at:
        if not args.quiet:
          print("Task {0} still running...".format(task_id), flush=True)
        timeleft = timeout_at - time.time()
        time.sleep(min(30, timeleft))
      else:
        raise Exception("Task {0} did not finish in time!".format(task_id))

def raise_friendly_error(e):
  if isinstance(e, HTTPError):
    text = e.read().decode('utf-8').strip()
    reresult = re.search(r"<pre>(.*?)</pre>", text, re.DOTALL)
    if reresult:
      text = reresult.group(1).strip()
    raise Exception("HTTP Error {0}: {1}, check overlord log for more details.\n{2}".format(e.code, e.reason, text))
  raise e

def await_load_completion(args, datasource, timeout_at, ctx=None):
  loopCount = 1
  while True:
    print("Wait for load completion number {0}", loopCount , flush=True)
    loopCount+=1
    url = args.coordinator_url.rstrip("/") + "/druid/coordinator/v1/loadstatus"
    req = Request(url)
    add_basic_auth_header(args, req)
    timeleft = timeout_at - time.time()
    response_timeout = min(max(timeleft, 5), 10)
    response = urlopen(req, None, response_timeout, context=ctx)
    response_obj = json.loads(response.read())
    load_status = response_obj.get(datasource, 0.0)
    if load_status >= 100.0:
      print("{0} loading complete! You may now query your data".format(datasource), flush=True)
      return
    else:
      if time.time() < timeout_at:
        if not args.quiet:
          print("{0} is {1}% finished loading...".format(datasource, load_status), flush=True)
        timeleft = timeout_at - time.time()
        time.sleep(min(5, timeleft))
      else:
        raise Exception("{0} was not loaded in time!".format(datasource))
